LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;

ENTITY binary_counter IS
    GENERIC (
      N   : natural := 10
   );
    PORT (
     async_res : IN  std_logic;
     async_ld  : IN  std_logic;
     clk       : IN  std_logic;

     sync_res  : IN  std_logic;
     sync_ld   : IN  std_logic;
     ld_val    : IN  std_logic_vector (N-1 DOWNTO 0);
     
     cnt_en    : IN  std_logic;
     cnt_up    : IN  std_logic;
     cnt_top   : IN  std_logic_vector (N-1 DOWNTO 0);

     cnt_max   : OUT std_logic;
     cnt_zero  : OUT std_logic;
     cnt_out   : OUT std_logic_vector (N-1 DOWNTO 0)
    );
END ENTITY binary_counter;

ARCHITECTURE rtl OF binary_counter IS
  SIGNAL cnt_nx     : unsigned (N-1 DOWNTO 0);
  SIGNAL cnt_d      : unsigned (N-1 DOWNTO 0);
  SIGNAL cnt_q      : unsigned (N-1 DOWNTO 0);
  SIGNAL cnt_max_i  : std_logic; 
  SIGNAL cnt_zero_i : std_logic; 
  SIGNAL increment  : std_logic;
  SIGNAL decrement  : std_logic;
BEGIN

  increment <= '1' WHEN cnt_en = '1' AND cnt_up = '1' ELSE '0';
  decrement <= '1' WHEN cnt_en = '1' AND cnt_up = '0' ELSE '0';

  cnt_nx  <= cnt_q+1 WHEN increment = '1' ELSE
             cnt_q-1 WHEN decrement = '1' ELSE
             cnt_q;

  cnt_max_i  <= '1' WHEN cnt_q >= unsigned(cnt_top) ELSE '0';
  cnt_zero_i <= '1' WHEN to_integer(cnt_q) = 0 ELSE '0';

  cnt_d   <= (OTHERS => '0')   WHEN sync_res   = '1' ELSE --synchronni reset ma prioritu pred citanim 
             unsigned(ld_val)  WHEN sync_ld    = '1' ELSE --synchronni preload ma prioritu pred citanim
             (OTHERS => '0')   WHEN cnt_max_i  = '1' AND increment = '1' ELSE --podteceni, kdyz citame dolu
             unsigned(cnt_top) WHEN cnt_zero_i = '1' AND decrement = '1' ELSE                           
             cnt_nx;    

  reg_cnt : PROCESS (clk, async_res, async_ld, ld_val)
  BEGIN
    IF async_res='1' THEN
      cnt_q <= (OTHERS => '0');
    ELSIF async_ld='1' THEN
      cnt_q <= unsigned(ld_val);
    ELSIF clk'EVENT AND clk='1' THEN
      cnt_q <= cnt_d;
    END IF;
  END PROCESS reg_cnt;

  cnt_max <= cnt_max_i;
  cnt_zero <= cnt_zero_i;
  cnt_out  <= std_logic_vector(cnt_q);
END ARCHITECTURE rtl;
