LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;


ARCHITECTURE rtl_gray OF cnt IS
  SIGNAL cnt_d     : std_logic_vector (N-1 DOWNTO 0);
  SIGNAL cnt_q     : std_logic_vector (N-1 DOWNTO 0);
  SIGNAL cnt_b     : std_logic_vector (N-1 DOWNTO 0);
  SIGNAL cnt_b_nxt : std_logic_vector (N-1 DOWNTO 0);

  COMPONENT gray2bin IS
    GENERIC (
	    N   : natural := 4
	 );
    PORT (
       gray : IN  std_logic_vector (N-1 DOWNTO 0);
       bin  : OUT std_logic_vector (N-1 DOWNTO 0)
    );
  END COMPONENT gray2bin;

  COMPONENT bin2gray IS
    GENERIC (
	    N   : natural := 4
	 );
    PORT (
       bin  : IN  std_logic_vector (N-1 DOWNTO 0);
       gray : OUT std_logic_vector (N-1 DOWNTO 0)
    );
  END COMPONENT bin2gray;

BEGIN
  reg_cnt : PROCESS (clk, res)
  BEGIN
      IF res='1' THEN
        cnt_q <= (OTHERS => '0');
      ELSIF clk'EVENT AND clk='1' THEN
        cnt_q <= cnt_d;
      END IF;
  END PROCESS reg_cnt;

  i_gray2bin:gray2bin
    GENERIC MAP (N => N)
    PORT    MAP (
       gray => cnt_q,
       bin  => cnt_b
    );

  cnt_b_nxt <= std_logic_vector(unsigned(cnt_b)+1);

  i_bin2gray:bin2gray
    GENERIC MAP (N => N)
    PORT    MAP (
       bin  => cnt_b_nxt,
       gray => cnt_d
    );

  cnt_out <= cnt_q;
END ARCHITECTURE rtl_gray;